#!/usr/bin/perl

##### {COPYRIGHT-TOP} ##############################################
# Licensed Materials - Property of IBM
# 5725-E59
#
# (c) Copyright IBM Corp. 2000, 2010 

# US Government Users Restricted Rights - Use, duplication, or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

# The following sample of source code ("Sample") is owned by International 
# Business Machines Corporation or one of its subsidiaries ("IBM") and is 
# copyrighted and licensed, not sold. You may copy and modify the Sample 
# for internal use only provided such use is within the limits of the license 
# rights under this Agreement, provided however that You may not alter or 
# delete any copyright information or notices contained in the Sample.
# 
# The Sample code is provided to you on an "AS IS" basis, without warranty of 
# any kind. IBM HEREBY EXPRESSLY DISCLAIMS ALL WARRANTIES, EITHER EXPRESS OR 
# IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Some jurisdictions do 
# not allow for the exclusion or limitation of implied warranties, so the above 
# limitations or exclusions may not apply to you. IBM shall not be liable for 
# any damages you suffer as a result of using, copying, modifying or 
# distributing the Sample, even if IBM has been advised of the possibility of 
# such damages.
# 
##### {COPYRIGHT-END} ##############################################

use strict;
use Getopt::Long;
use File::Find;
use Pim::EnvConfig;
use Pim::Localizer;
use Pim::Service::AppServer;
use Pim::Utils qw(getDirDelimiter platformIsWindows getUsername $pathSeparator promptAndReadPassword);
use Config;


die "ERROR: Symlinks are not implemented in Perl on this platform\n" unless (defined $Config{d_symlink});

sub wanted();
sub installWar($$);

our $verbose = 0;
my $silent = 0;
our $wsadminPwd = '';
our $wsadminUser = '';
our $security = '';

GetOptions("verbose" => \$verbose, "silent" => \$silent, "wsadminPwd=s" => \$wsadminPwd, "wsadminUsername=s" => \$wsadminUser);

my $envConfig = Pim::EnvConfig->new();
my $localizer = Pim::Localizer->new();

my $sec = $envConfig->isWebSphereSecuritySet();
our $top = $envConfig->getTop();

if (platformIsWindows()) {
	print "Su\n";
	exit;
}

if ($envConfig->getAppSvrType() !~ /websphere/i) {
	print "Error: This script can only be used in WebSphere installations. This installation is configured for " . $envConfig->getAppSvrType() . "\n";
	exit;
}

print "Installing web service WAR file for IBM InfoSphere Master Data Management Collaboration Server\n";

if (($silent) and ($sec)) {
	if ($wsadminUser eq '' or $wsadminPwd eq '') {
		print "Error: The password or username for wsadmin.sh is not set.\n";
		print "When using the --silent option the username and password for wsadmin.sh must be supplied\n";
		print "Please supply the username with --wsadminUsername and password with --wsadminPwd\n";
		exit;
	}
}

do {
	if ($sec) {
		print "You have indicated that WAS security is enabled.  Please provide the following authentication information:\n";
		if ($wsadminUser eq '') {
			print "WAS Username: ";
			$wsadminUser = <STDIN>;
		}
		if ($wsadminPwd eq '') {
			$wsadminPwd = promptAndReadPassword();
		}
		chomp $wsadminPwd;
		chomp $wsadminUser;
		
		$security = "-user $wsadminUser -password $wsadminPwd";
		
	}
} unless ($silent);

print "\n\n";

# Loop through the number of app servers defined and use the appservers that DO NOT have 'gds' in their name.
# This is to avoid installing MDMPIM on app servers defined for GDS.
my @appsvrs;
foreach my $appserver ($envConfig->getSvcNamesForType('appsvr')) {
    my $result = index($appserver, "gds");
    if($result == -1) {
        push @appsvrs, $appserver;
    }
}
print "Found " . scalar(@appsvrs) . " app server(s): " . join(",", @appsvrs) . "\n";

foreach my $fullName (@appsvrs) {
	my $appsvr = Pim::Service::AppServer->new( {fullName => $fullName} );
	
	my $cfg = $appsvr->getConfig({verbose => $verbose});
	my $appName = "ccd_$cfg->{appserver_name}_ws";
	
	print "App Server configuration for $fullName:\n";
	print "\tCell: $cfg->{cell_name}\n";
	print "\tNode name: $cfg->{node_name}\n";
	print "\tProfile name: $cfg->{application_server_profile}\n";
	print "\tApp server port: $cfg->{port}\n";
	print "\tJVM memory settings: $cfg->{mem_settings}\n";
	print "\tWebSphere application server name: $cfg->{appserver_name}\n";
	print "\tJVM arguments: $cfg->{jvm_opts}\n\n";	
	unless ($silent) {
		print "Installing web service WAR file on Application Server: $cfg->{appserver_name} on node $cfg->{node_name}\n";
		print "Warning!  Running this script will erase any previously configured WebSphere application component named '$appName'!\n";
		print "\n";
		print "Are you sure you want to do this? [y/n] ";
		my $res = <STDIN>;
		chomp $res;
		exit unless (uc $res eq 'Y');	
	}
	installWar($cfg, $appName);
	$appsvr->saveCurrentWASConfig();			# initial save.  This is used by Pim::Service::AppServer in WAS configurations
}

sub installWar($$) {
	my ($cfg, $appName) = @_;
	
	my $warFile = "$top/samples/webservices/webservices.samples.war";
	my $jacl = "$top/samples/webservices/deploy/websphere/install_ws.jacl";
	my $wsadmin = $envConfig->getWsAdminExe();
	
	my $cmd = "$wsadmin $security ";
	$cmd .= "-f $jacl ";
	$cmd .= "$cfg->{node_name} ";
	$cmd .= "$cfg->{appserver_name} ";
	$cmd .= "$warFile ";
	$cmd .= "$appName ";
	$cmd .= "$cfg->{vhost_name} ";
           
	print "\nExecuting '$cmd'\n" if ($verbose);
	print "\n<---- Output\n\n";	
	open CMD, "$cmd |" or die "Cannot start '$cmd': $!";
	while (<CMD>) {
		print;
	}
	close CMD;
	print "end output--->\n\n";
}

